/*****************************************************************************/
/*                                                                           */
/* Title: Hidden Markov Model evaluation (Baum-Welch)                        */
/* File:  "hmm_eval.c"                                                       */
/*                                                                           */
/* Author: Michael C. Nechyba                                                */
/* Last modified: 5/6/96                                                     */
/*                                                                           */
/*****************************************************************************/

#include <stdio.h>
#include <math.h>
#include <stdlib.h>
#include <string.h>
#include "matrix.h"
#include "file_io.h"
#include "gen_math.h"
#include "hmm_util.h"
#ifdef MacOS
#include <console.h>
#endif
#include "arg.h"

#define PROB_MAX 1000000

double prob[PROB_MAX];

/**************************** Main Program ***********************************/

int main(int argc, char **argv)
{
  MULTIPLE m;
  HMM *model;
  char *in_file = NULL, *hmm_file = NULL, format[STR_REG];
  double floor = 0.000000001, norm_prob;
  int precision = 15, scale = 1, i;

#ifdef MacOS
  argc = ccommand(&argv);
#endif

  arg_parse(argc, argv,
	    "", "Usage:",
	    "-i %S", &in_file,  "set input file",
	    "-h %S", &hmm_file, "set hmm file",
	    "-f [%F]", &floor, "set minimum HMM entry [%.15f]", floor,
	    "-p [%d]", &precision, "set precision [%d]", precision,
	    "-s [%d]", &scale, "use scaling? [%d]", scale,
	    NULL);

  if(in_file == NULL || hmm_file == NULL) {
    fprintf(stderr, "No input and/or output file specified.\n");
    exit(-1);
  }

  model = read_hmm(model, hmm_file);
  model = bw_smooth(model, floor, 1);
  read_data_file(&m, in_file);

  sprintf(format, "%%s %%d %%.%df %%.%df (%%s)\n", precision, precision);

  norm_prob = exp10(probability(model, m.obs, m.n_sequence, scale,
				&prob[0])/((double) m.n_total));
  fprintf(stdout, format, "Total", m.n_total, norm_prob, model->prob, 
	  hmm_file);

  for(i = 0; i < m.n_sequence; i++)
    fprintf(stdout, format, 
	    m.obs[i]->name, m.obs[i]->T,
	    exp10(prob[i]/((double) m.obs[i]->T)), prob[i], hmm_file);
}
