/*****************************************************************************/
/*                                                                           */
/* Title: Hidden Markov Model sequence generation                            */
/* File: "hmm_gen.c"                                                         */
/*                                                                           */
/* Author: Michael C. Nechyba                                                */
/* Last modified: 5/9/96                                                     */
/*                                                                           */
/*****************************************************************************/

#include <stdio.h>
#include <math.h>
#include <stdlib.h>
#include <string.h>
#include "matrix.h"
#include "file_io.h"
#include "gen_math.h"
#include "hmm_util.h"
#ifdef MacOS
#include <console.h>
#endif
#include "arg.h"

/**************************** Main Program ***********************************/

int main(int argc, char **argv)
{
  char *in_file = NULL;
  int i, n = 10, seed = 0;
  HMM *model;
  SEQUENCE obs;

  obs.T = 10;

#ifdef MacOS
  argc = ccommand(&argv);
#endif

  arg_parse(argc, argv,
	    "", "Usage:",
	    "-i %S", &in_file, "set input file",
	    "-n [%d]", &obs.T,  "set vec length [%d]", obs.T,
	    "-s [%d]", &seed, "random seed [%d]", seed,
	    NULL);

  if(in_file == NULL) {
    fprintf(stderr, "No input hmm file specified. Exiting...\n");
    exit(-1);
  }

  model = read_hmm(model, in_file);

  fprintf(stdout, "1 %d\n", model->n_observables);
  fprintf(stdout, "%d %s\n", obs.T, in_file);

  generate_sequence(model, &obs, seed);

  for(i = 0; i < obs.T; i++)
    fprintf(stdout, "%d\n", obs.sequence[i]);
}
