/*****************************************************************************/
/*                                                                           */
/* Title: Hidden Markov Model Viterbi algorithm                              */
/* File:  "hmm_viterbi.c"                                                    */
/*                                                                           */
/* Author: Michael C. Nechyba                                                */
/* Last modified: 11/8/2002                                                  */
/*                                                                           */
/*****************************************************************************/

#include <stdio.h>
#include <math.h>
#include <stdlib.h>
#include <string.h>
#include "matrix.h"
#include "file_io.h"
#include "gen_math.h"
#include "hmm_util.h"
#ifdef MacOS
#include <console.h>
#endif
#include "arg.h"

#define PROB_MAX 1000000

double prob[PROB_MAX];

/**************************** Main Program ***********************************/

int main(int argc, char **argv)
{
  MULTIPLE m;
  HMM *model;
  char *in_file = NULL, *hmm_file = NULL, format[STR_REG];
  double floor = 0.000000001, log_prob, norm_prob;
  int precision = 15, i, j, Tmax;
  SEQUENCE *q;

#ifdef MacOS
  argc = ccommand(&argv);
#endif

  arg_parse(argc, argv,
	    "", "Usage:",
	    "-i %S", &in_file,  "set input file",
	    "-h %S", &hmm_file, "set hmm file",
	    "-f [%F]", &floor, "set minimum HMM entry [%.15f]", floor,
	    "-p [%d]", &precision, "set precision [%d]", precision,
	    NULL);

  if(in_file == NULL || hmm_file == NULL) {
    fprintf(stderr, "No input and/or output file specified.\n");
    exit(-1);
  }

  model = read_hmm(model, hmm_file);
  model = bw_smooth(model, floor, 1);
  read_data_file(&m, in_file);
  //display_hmm(stdout, model, 15);

  Tmax = -1;
  for(i = 0; i < m.n_sequence; i++) {
    if(m.obs[i]->T > Tmax)
      Tmax = m.obs[i]->T;
  }
  q = allocate_sequence(q, Tmax);

  sprintf(format, "%%s %%.%df %%.%df %%d -", precision, precision);

  for(i = 0; i < m.n_sequence; i++) {
    log_prob = viterbi(q, model, m.obs[i]);
    norm_prob = exp10(log_prob/((double) m.obs[i]->T));
        fprintf(stdout, format, 
		m.obs[i]->name, 
		norm_prob, log_prob, m.obs[i]->T);
    for(j = 0; j < m.obs[i]->T; j++)
      fprintf(stdout, " %d", q->sequence[j]);
    fprintf(stdout, "\n");
  }

  exit(1);
}
